(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory(require('styled-components'), require('ramda')) :
	typeof define === 'function' && define.amd ? define(['styled-components', 'ramda'], factory) :
	(global.styledMQ = factory(global.styled,global.R));
}(this, (function (styledComponents,ramda) { 'use strict';

function preserveCamelCase(str) {
	let isLastCharLower = false;
	let isLastCharUpper = false;
	let isLastLastCharUpper = false;

	for (let i = 0; i < str.length; i++) {
		const c = str[i];

		if (isLastCharLower && /[a-zA-Z]/.test(c) && c.toUpperCase() === c) {
			str = str.substr(0, i) + '-' + str.substr(i);
			isLastCharLower = false;
			isLastLastCharUpper = isLastCharUpper;
			isLastCharUpper = true;
			i++;
		} else if (isLastCharUpper && isLastLastCharUpper && /[a-zA-Z]/.test(c) && c.toLowerCase() === c) {
			str = str.substr(0, i - 1) + '-' + str.substr(i - 1);
			isLastLastCharUpper = isLastCharUpper;
			isLastCharUpper = false;
			isLastCharLower = true;
		} else {
			isLastCharLower = c.toLowerCase() === c;
			isLastLastCharUpper = isLastCharUpper;
			isLastCharUpper = c.toUpperCase() === c;
		}
	}

	return str;
}

var camelcase = function (str) {
	if (arguments.length > 1) {
		str = Array.from(arguments)
			.map(x => x.trim())
			.filter(x => x.length)
			.join('-');
	} else {
		str = str.trim();
	}

	if (str.length === 0) {
		return '';
	}

	if (str.length === 1) {
		return str.toLowerCase();
	}

	if (/^[a-z0-9]+$/.test(str)) {
		return str;
	}

	const hasUpperCase = str !== str.toLowerCase();

	if (hasUpperCase) {
		str = preserveCamelCase(str);
	}

	return str
		.replace(/^[_.\- ]+/, '')
		.toLowerCase()
		.replace(/[_.\- ]+(\w|$)/g, (m, p1) => p1.toUpperCase());
};

var isArray = Array.isArray || function (obj) {
  return Object.prototype.toString.call(obj) === '[object Array]';
};

var isDate = function (obj) {
  return Object.prototype.toString.call(obj) === '[object Date]';
};

var isRegex = function (obj) {
  return Object.prototype.toString.call(obj) === '[object RegExp]';
};

var has$1 = Object.prototype.hasOwnProperty;
var objectKeys = Object.keys || function (obj) {
  var keys$$1 = [];
  for (var key in obj) {
    if (has$1.call(obj, key)) {
      keys$$1.push(key);
    }
  }
  return keys$$1;
};

function dashCase(str) {
  return str.replace(/[A-Z](?:(?=[^A-Z])|[A-Z]*(?=[A-Z][^A-Z]|$))/g, function (s, i) {
    return (i > 0 ? '-' : '') + s.toLowerCase();
  });
}

function map$1(xs, f) {
  if (xs.map) {
    return xs.map(f);
  }
  var res = [];
  for (var i = 0; i < xs.length; i++) {
    res.push(f(xs[i], i));
  }
  return res;
}

function reduce(xs, f, acc) {
  if (xs.reduce) {
    return xs.reduce(f, acc);
  }
  for (var i = 0; i < xs.length; i++) {
    acc = f(acc, xs[i], i);
  }
  return acc;
}

function walk(obj) {
  if (!obj || typeof obj !== 'object') {
    return obj;
  }
  if (isDate(obj) || isRegex(obj)) {
    return obj;
  }
  if (isArray(obj)) {
    return map$1(obj, walk);
  }
  return reduce(objectKeys(obj), function (acc, key) {
    var camel = dashCase(key);
    acc[camel] = walk(obj[key]);
    return acc;
  }, {});
}

var dasherize = function (obj) {
  if (typeof obj === 'string') {
    return dashCase(obj);
  }
  return walk(obj);
};

var MEDIA_PREFIX = '@media';

// -----------------------------------------------------------------------------
// CONFIG
// -----------------------------------------------------------------------------

var SEPARATOR_VALUES = Object.freeze({
  rem: 0.01,
  em: 0.01,
  px: 1,
  dpi: 1
});

var DIMENSIONS_UNITS = Object.freeze({
  EM: 'em',
  REM: 'rem',
  PX: 'px'
});

var RESOLUTION_UNIT = 'dpi';

var UNITS = Object.freeze({
  DIMENSIONS: DIMENSIONS_UNITS,
  RESOLUTION: Object.freeze({ DPI: 'dpi' })
});

var BREAKPOINT_MAP_NAMES = Object.freeze(['width', 'height', 'resolution']);

// -----------------------------------------------------------------------------
// MEDIA TYPES
// -----------------------------------------------------------------------------

var MEDIA_TYPES = Object.freeze({
  ALL: 'all',
  PRINT: 'print',
  SCREEN: 'screen',
  SPEECH: 'speech'
});

var defineProperty = function (obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
};

var _extends = Object.assign || function (target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }

  return target;
};





















var slicedToArray = function () {
  function sliceIterator(arr, i) {
    var _arr = [];
    var _n = true;
    var _d = false;
    var _e = undefined;

    try {
      for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
        _arr.push(_s.value);

        if (i && _arr.length === i) break;
      }
    } catch (err) {
      _d = true;
      _e = err;
    } finally {
      try {
        if (!_n && _i["return"]) _i["return"]();
      } finally {
        if (_d) throw _e;
      }
    }

    return _arr;
  }

  return function (arr, i) {
    if (Array.isArray(arr)) {
      return arr;
    } else if (Symbol.iterator in Object(arr)) {
      return sliceIterator(arr, i);
    } else {
      throw new TypeError("Invalid attempt to destructure non-iterable instance");
    }
  };
}();



var taggedTemplateLiteral = function (strings, raw) {
  return Object.freeze(Object.defineProperties(strings, {
    raw: {
      value: Object.freeze(raw)
    }
  }));
};









var toConsumableArray = function (arr) {
  if (Array.isArray(arr)) {
    for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) arr2[i] = arr[i];

    return arr2;
  } else {
    return Array.from(arr);
  }
};

var separatorValueForUnit = function separatorValueForUnit(unit) {
  return SEPARATOR_VALUES[unit];
};

var appendUnit = function appendUnit(value, unit) {
  return ramda.join('', [value, unit]);
};

var unitIsRemOrEm = ramda.contains(ramda.__, [UNITS.DIMENSIONS.EM, UNITS.DIMENSIONS.REM]);

var pxToRemOrEmValue = function pxToRemOrEmValue(value, baseFontSize) {
  return ramda.divide(value, baseFontSize);
};

var remOrEmToPxValue = function remOrEmToPxValue(value, baseFontSize) {
  return ramda.multiply(value, baseFontSize);
};

var toDimensionOutput = function toDimensionOutput(unit, baseFontSize, value) {
  return appendUnit(unitIsRemOrEm(unit) ? pxToRemOrEmValue(value, baseFontSize) : value, unit);
};

var elementsOfUnitedNumber = function elementsOfUnitedNumber(value) {
  var captures = /^(-?\d+(?:.\d+)?)([a-z]+)?$/.exec(value);
  if (ramda.none(ramda.complement(ramda.isNil), [captures, captures[1], captures[2]])) {
    throw new Error('You can\'t get the numeric portion of \'' + value + '\'');
  }
  return [Number(captures[1]), captures[2]];
};

var numericPartOfUnitedNumber = ramda.compose(ramda.view(ramda.lensIndex(0)), elementsOfUnitedNumber);

var unitPartOfUnitedNumber = ramda.compose(ramda.view(ramda.lensIndex(1)), elementsOfUnitedNumber);

var unitedDimensionToUnitlessPixelValue = function unitedDimensionToUnitlessPixelValue(value, baseFontSize) {
  var _elementsOfUnitedNumb = elementsOfUnitedNumber(value),
      _elementsOfUnitedNumb2 = slicedToArray(_elementsOfUnitedNumb, 2),
      number = _elementsOfUnitedNumb2[0],
      unit = _elementsOfUnitedNumb2[1];

  return unitIsRemOrEm(unit) ? remOrEmToPxValue(number, baseFontSize) : number;
};

var unitedResolutionToUnitlessValue = function unitedResolutionToUnitlessValue(value) {
  return numericPartOfUnitedNumber(value);
};

var isBoolean = ramda.is(Boolean);
var isNumber = ramda.both(ramda.is(Number), ramda.complement(ramda.equals(NaN)));
var isObject = ramda.is(Object);
var isString = ramda.is(String);
var isArray$1 = ramda.is(Array);
var isNull = function isNull(value) {
  return value === null;
};
var isUndefined = function isUndefined(value) {
  return value === undefined;
};

var isPopulatedObject = ramda.both(ramda.complement(ramda.isEmpty), isObject);
var isRatioString = ramda.test(/^[1-9]+[0-9]* ?\/ ?[1-9]+[0-9]*$/);
var isPositiveNumber = ramda.both(isNumber, ramda.gt(ramda.__, 0));
var isPositiveNumberOrZero = ramda.both(isNumber, ramda.gte(ramda.__, 0));
var isPositiveInteger = ramda.both(isPositiveNumber, Number.isInteger);
var isPositiveIntegerOrZero = ramda.both(isPositiveNumberOrZero, Number.isInteger);
var isNumberWithUnit = ramda.curry(function (units, value) {
  var regex = '^-?\\d+(?:.\\d+)?(?:' + ramda.join('|', units) + ')$';
  return new RegExp(regex).test(value);
});
var isNumericPartOfUnitValuePositive = ramda.compose(isPositiveNumber, numericPartOfUnitedNumber);
var isPositiveNumberWithPixelUnit = ramda.both(isNumberWithUnit([DIMENSIONS_UNITS.PX]), isNumericPartOfUnitValuePositive);
var isNumberWithDimensionsUnit = isNumberWithUnit(ramda.values(DIMENSIONS_UNITS));
var isNumberWithResolutionUnit = isNumberWithUnit([RESOLUTION_UNIT]);
var isPositiveNumberWithResolutionUnit = ramda.both(isNumberWithResolutionUnit, isNumericPartOfUnitValuePositive);
var isPositiveNumberWithDimensionsUnit = ramda.both(isNumberWithDimensionsUnit, isNumericPartOfUnitValuePositive);
var doesListIncludeValue = function doesListIncludeValue(list) {
  return ramda.contains(ramda.__, ramda.values(list));
};
var isDimensionsUnitValid = ramda.contains(ramda.__, ramda.values(DIMENSIONS_UNITS));
var isMediaTypeValid = ramda.flip(ramda.contains)(ramda.values(MEDIA_TYPES));
var areMediaTypesValid = ramda.both(ramda.all(isMediaTypeValid), ramda.complement(ramda.isEmpty));

var toUnit = function toUnit(dimensionsUnit, baseFontSize) {
  return ramda.partial(toDimensionOutput, [dimensionsUnit, baseFontSize]);
};

var prepareUnitlessValue = function prepareUnitlessValue(value, shouldSeparateQueries, dimensionsUnit) {
  return ramda.when(ramda.always(shouldSeparateQueries), ramda.subtract(ramda.__, separatorValueForUnit(dimensionsUnit)))(value);
};

var dimensionsValueRenderer = (function () {
  var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      _ref$baseFontSize = _ref.baseFontSize,
      baseFontSize = _ref$baseFontSize === undefined ? 16 : _ref$baseFontSize,
      _ref$dimensionsUnit = _ref.dimensionsUnit,
      dimensionsUnit = _ref$dimensionsUnit === undefined ? UNITS.DIMENSIONS.EM : _ref$dimensionsUnit,
      _ref$shouldSeparateQu = _ref.shouldSeparateQueries,
      shouldSeparateQueries = _ref$shouldSeparateQu === undefined ? true : _ref$shouldSeparateQu;

  return function (value, shouldSeparate) {
    if (isNumberWithDimensionsUnit(value)) {
      value = unitedDimensionToUnitlessPixelValue(value, baseFontSize);
    }
    var preparedValue = prepareUnitlessValue(value, shouldSeparateQueries && shouldSeparate, dimensionsUnit);
    var x = toUnit(dimensionsUnit, baseFontSize)(preparedValue);
    return x;
  };
});

var prepareUnitlessValue$1 = function prepareUnitlessValue(value, shouldSeparateQueries, unit) {
  return ramda.when(ramda.always(shouldSeparateQueries), ramda.subtract(ramda.__, separatorValueForUnit(unit)))(value);
};

var resolutionValueRenderer = (function () {
  var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      _ref$shouldSeparateQu = _ref.shouldSeparateQueries,
      shouldSeparateQueries = _ref$shouldSeparateQu === undefined ? true : _ref$shouldSeparateQu;

  return function (value, shouldSeparate) {
    if (isNumberWithResolutionUnit(value)) {
      value = unitedResolutionToUnitlessValue(value);
    }

    var preparedValue = prepareUnitlessValue$1(value, shouldSeparateQueries && shouldSeparate, UNITS.RESOLUTION.DPI);

    return appendUnit(preparedValue, UNITS.RESOLUTION.DPI);
  };
});

var aspectRatioValueRenderer = (function () {
  return ramda.identity;
});

var colorValueRenderer = (function () {
  return ramda.identity;
});

var monochromeValueRenderer = (function () {
  return ramda.identity;
});

var dimensionValidator = {
  message: "You must supply a 'dimension' as either a unitless positive number or a string comprised of a number followed by em, rem px",
  validate: ramda.either(isPositiveNumber, isPositiveNumberWithDimensionsUnit)
};

var resolutionValidator = {
  message: "You must supply 'resolution' as a positive number",
  validate: ramda.either(isNumber, isPositiveNumberWithResolutionUnit)
};

var aspectRatioValidator = {
  message: "You must supply an 'aspectRatio' in the form of two positive integers separated by a forward slash, for example '16/9'",
  validate: isRatioString
};

var colorValidator = {
  message: "You must supply 'colorIndex' as zero or a positive integer",
  validate: ramda.either(isPositiveIntegerOrZero, isNull)
};

var monochromeValidator = {
  message: "You must supply 'monochrome' as zero or a postive integer",
  validate: ramda.either(isPositiveIntegerOrZero, isNull)
};

var ORIENTATION = Object.freeze(['portrait', 'landscape']);
var SCAN = Object.freeze(['interlace', 'progressive']);
var GRID = Object.freeze([0, 1]);
var UPDATE = Object.freeze(['none', 'slow', 'fast']);
var OVERFLOW_BLOCK = Object.freeze(['none', 'scroll', 'optional-paged']);
var OVERFLOW_INLINE = Object.freeze(['none', 'scroll']);
var COLOR_GAMUT = Object.freeze(['srgb', 'p3', 'rec2020']);
var DISPLAY_MODE = Object.freeze(['fullscreen', 'standalone', 'minimal-ui', 'browser']);

var LINEAR_FEATURES = [{
  name: 'orientation',
  validValues: ORIENTATION
}, {
  name: 'scan',
  validValues: SCAN
}, {
  name: 'grid',
  validValues: GRID,
  allowNoArgument: true
}, {
  name: 'update',
  validValues: UPDATE,
  allowNoArgument: true
}, {
  name: 'overflow-block',
  validValues: OVERFLOW_BLOCK
}, {
  name: 'overflow-inline',
  validValues: OVERFLOW_INLINE
}, {
  name: 'color-gamut',
  validValues: COLOR_GAMUT
}, {
  name: 'display-mode',
  validValues: DISPLAY_MODE
}];

var RANGED_FEATURES = [{
  name: 'width',
  valueRenderer: dimensionsValueRenderer,
  validator: dimensionValidator
}, {
  name: 'height',
  valueRenderer: dimensionsValueRenderer,
  validator: dimensionValidator
}, {
  name: 'resolution',
  valueRenderer: resolutionValueRenderer,
  validator: resolutionValidator
}, {
  name: 'aspect-ratio',
  valueRenderer: aspectRatioValueRenderer,
  validator: aspectRatioValidator
}, {
  name: 'color',
  valueRenderer: colorValueRenderer,
  validator: colorValidator,
  config: {
    allowNoArgument: true
  }
}, {
  name: 'color-index',
  valueRenderer: colorValueRenderer,
  validator: colorValidator,
  config: {
    allowNoArgument: true
  }
}, {
  name: 'monochrome',
  valueRenderer: monochromeValueRenderer,
  validator: monochromeValidator,
  config: {
    allowNoArgument: true
  }
}];

var rangedFeatureNames = ramda.map(ramda.compose(camelcase, ramda.prop('name')))(RANGED_FEATURES);

var rangedFeatureNamed = ramda.compose(ramda.find(ramda.__, RANGED_FEATURES), ramda.propEq('name'), dasherize);

// eslint-disable-next-line import/prefer-default-export
var ensureArray = ramda.when(ramda.complement(isArray$1), function (mediaTypes) {
  return [mediaTypes];
});

// eslint-disable-next-line import/prefer-default-export
var toCommaSeparatedList = function toCommaSeparatedList(values$$1) {
  return values$$1.join(', ');
};

var wrapWithQuotes = ramda.map(function (v) {
  return '\'' + v + '\'';
});
var keysToCommaSeparatedList = ramda.compose(toCommaSeparatedList, wrapWithQuotes, ramda.keys);
var valuesToCommaSeparatedList = ramda.compose(toCommaSeparatedList, ramda.values);
var objectToString = JSON.stringify;

// Horrible hackery to get round issues with Babel extending builtins.
// This is the only way to have a custom error.
var InvalidValueError = function InvalidValueError(message) {
  var error = new Error(message);
  error.name = 'InvalidValueError';
  this.name = error.name;
  this.message = error.message;
  if (error.stack) this.stack = error.stack;
  this.toString = function () {
    return this.name + ': ' + this.message;
  };
};
InvalidValueError.prototype = new Error();
InvalidValueError.prototype.name = 'InvalidValueError';

var throwError = function throwError(message) {
  throw new InvalidValueError(message);
};

var composeError = function composeError(message) {
  return ramda.compose(throwError, message);
};

var invalidBreakpointsErrorMessage = function invalidBreakpointsErrorMessage(breakpoints) {
  return 'Breakpoints must be an object, but you supplied \'' + objectToString(breakpoints) + '\'.';
};

var emptyBreakpointMapErrorMessage = function emptyBreakpointMapErrorMessage(breakpointMap) {
  return '\n  You must not supply an empty object of breakpoints to \'configure()\', but the you supplied \'' + objectToString(breakpointMap) + '\'.';
};

var invalidBreakpointNamesErrorMessage = function invalidBreakpointNamesErrorMessage(breakpointMap) {
  return 'You supplied a breakpoint set with an invalid name. Valid values are: \'' + rangedFeatureNames + '\', but you supplied: \'' + objectToString(breakpointMap) + '\'.';
};

var emptyBreakpointSetErrorMessage = function emptyBreakpointSetErrorMessage(breakpointMapName) {
  return 'A breakpoint set must contain at least one breakpoint, but you supplied an empty breakpoint set for the \'' + objectToString(breakpointMapName) + '\' map.';
};

var mssingBreakpointMapErrorMessage = function mssingBreakpointMapErrorMessage(name) {
  return 'This mq object was not configured with a breakpoint set for \'' + name + '\'.';
};

var missingBreakpointErrorMessage = ramda.curry(function (name, breakpointMapName, breakpoints) {
  return 'There is no \'' + breakpointMapName + '\' breakpoint defined called \'' + name + '\', only: ' + keysToCommaSeparatedList(breakpoints) + ' are defined.';
});

var sameBreakpointsForBetweenErrorMessage = function sameBreakpointsForBetweenErrorMessage(name) {
  return 'You must supply two different breakpoints to \'widthBetween\' but both were: \'' + name + '\'.';
};

var invalidMediaTypeErrorMessage = function invalidMediaTypeErrorMessage(value) {
  return '\'mediaType\' must be one of \'' + valuesToCommaSeparatedList(MEDIA_TYPES) + '\' but you supplied: \'' + value + '\'.';
};

var invalidBreakpointSetValueErrorMessage = ramda.curry(function (message, breakpoints) {
  return message + ' but you supplied \'' + objectToString(breakpoints) + '\'.';
});

var invalidBaseFontSizeErrorMessage = function invalidBaseFontSizeErrorMessage(value) {
  return '\'baseFontSize\' must be a positive number, but you supplied \'' + value + '\'.';
};

var invalidDefaultMediaTypeErrorMessage = function invalidDefaultMediaTypeErrorMessage(value) {
  return '\'defaultMediaType\' must be one of \'' + valuesToCommaSeparatedList(MEDIA_TYPES) + '\' but was \'' + value + '\'.';
};

var invalidDimensionsUnitErrorMessage = function invalidDimensionsUnitErrorMessage(value) {
  return '\'unit\' must be one of \'' + valuesToCommaSeparatedList(UNITS.DIMENSIONS) + '\' but was \'' + value + '\'.';
};

var shouldSeparateQueriesErrorMessage = function shouldSeparateQueriesErrorMessage(value) {
  return '\'shouldSeparateQueries\' must be a boolean but was \'' + value + '\'.';
};

var invalidFeatureErrorMessage = ramda.curry(function (name, possibleValues, value) {
  return '\n  \'' + name + '\' must be one of: \'' + possibleValues + '\' but was: \'' + value + '\'.';
});

var queryNoElementsErrorMessage = function queryNoElementsErrorMessage() {
  return "You must supply at least one argument to 'query()' to build a valid media query.";
};

var notNoElementsErrorMessage = function notNoElementsErrorMessage() {
  return "You must supply at least one argument to 'not()' to build a valid media query.";
};

var queryElementIsValidTypeErrorMessage = function queryElementIsValidTypeErrorMessage(value) {
  return 'You must only supply strings or arrays to \'query()\' but you supplied \'' + value + '\'.';
};

var queryChildElementIsValidTypeErrorMessage = function queryChildElementIsValidTypeErrorMessage(value) {
  return 'You must only supply strings or arrays as children of arrays passed in to \'query()\' but you supplied \'' + objectToString(value) + '\'.';
};

var queryNoNestedArraysErrorMessage = function queryNoNestedArraysErrorMessage(value) {
  return 'You must not supply any nested arrays as part of a query but you supplied \'' + value + '\'.';
};

var noUntweakedErrorMessage = function noUntweakedErrorMessage() {
  return 'There is no untweaked mq object available.';
};

// TODO: Can't move these to predicates as they import helper from features and
// this causes an acyclical dependency between features -> validators ->
// predicates.
var isBreakpointSetNameValid = ramda.contains(ramda.__, rangedFeatureNames);
var areBreakpointSetNamesValid = ramda.all(isBreakpointSetNameValid);

var validate = function validate(predicate, errorMessage) {
  return function (value) {
    ramda.unless(predicate, composeError(errorMessage))(value);
  };
};

// Validate a map of breakpoint sets.

// -----------------------------------------------------------------------------
// Exports
// -----------------------------------------------------------------------------

var validateBreakpointSetValues = function validateBreakpointSetValues(name, breakpointSet) {
  var _rangedFeatureNamed = rangedFeatureNamed(name),
      validator = _rangedFeatureNamed.validator;

  validate(ramda.compose(ramda.all(validator.validate), ramda.values), ramda.compose(ramda.curry(invalidBreakpointSetValueErrorMessage)(validator.message), ramda.values))(breakpointSet);
};

var validateBreakpointSetObject = function validateBreakpointSetObject(name, breakpointSet) {
  validate(isPopulatedObject, ramda.always(emptyBreakpointSetErrorMessage(name)))(breakpointSet);
};

// Validate a set of breakpoints.
var validateBreakpointSet = function validateBreakpointSet(name, breakpointSet) {
  validateBreakpointSetObject(name, breakpointSet);
  validateBreakpointSetValues(name, breakpointSet);
};

var validateMediaTypes = validate(areMediaTypesValid, invalidMediaTypeErrorMessage);

var validateBreakpointSetNames = validate(ramda.compose(areBreakpointSetNamesValid, ramda.keys), invalidBreakpointNamesErrorMessage);

var validateBreakpointObject = function validateBreakpointObject(breakpointMap) {
  validate(ramda.both(ramda.complement(isArray$1), isObject), invalidBreakpointsErrorMessage)(breakpointMap);
  validate(isPopulatedObject, emptyBreakpointMapErrorMessage)(breakpointMap);
};

var validateBreakpointMap = function validateBreakpointMap(breakpoints) {
  validateBreakpointObject(breakpoints);
  validateBreakpointSetNames(breakpoints);
};

var validateBreakpointSets = ramda.compose(ramda.map(function (_ref) {
  var _ref2 = slicedToArray(_ref, 2),
      name = _ref2[0],
      value = _ref2[1];

  return validateBreakpointSet(name, value);
}), ramda.toPairs);

var validateConfig = function validateConfig(_ref3) {
  var baseFontSize = _ref3.baseFontSize,
      defaultMediaType = _ref3.defaultMediaType,
      dimensionsUnit = _ref3.dimensionsUnit,
      shouldSeparateQueries = _ref3.shouldSeparateQueries;

  validate(ramda.either(isPositiveNumber, isPositiveNumberWithPixelUnit), invalidBaseFontSizeErrorMessage)(baseFontSize);

  validate(ramda.anyPass([isNull, ramda.compose(areMediaTypesValid, ensureArray)]), invalidDefaultMediaTypeErrorMessage)(defaultMediaType);

  validate(isDimensionsUnitValid, invalidDimensionsUnitErrorMessage)(dimensionsUnit);
  validate(isBoolean, shouldSeparateQueriesErrorMessage)(shouldSeparateQueries);
};

var validateFeature = function validateFeature(name, possibleValues, value) {
  validate(doesListIncludeValue(possibleValues), invalidFeatureErrorMessage(name, possibleValues))(value);
};

var buildMediaType = (function (defaultMediaType) {
  return function () {
    var mediaTypes = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [defaultMediaType];

    var mediaTypesArray = ensureArray(mediaTypes);
    validateMediaTypes(mediaTypesArray);
    return toCommaSeparatedList(mediaTypesArray);
  };
});

var nand = ramda.complement(ramda.and);
var nor = ramda.complement(ramda.or);
var notBoth = ramda.compose(ramda.complement, ramda.both);
var neither = ramda.compose(ramda.complement, ramda.either);
var notAllPass = ramda.compose(ramda.complement, ramda.allPass);
var nonePass = ramda.compose(ramda.complement, ramda.anyPass);

var nameValue = ramda.compose(ramda.join(': '), ramda.reject(ramda.isNil));
var isArrayOrString = ramda.either(isArray$1, isString);
var isNegationObject = ramda.both(isObject, ramda.has('not'));

var joinAnd = ramda.join(' and ');
var joinComma = ramda.join(', ');
var prefixWithNot = ramda.concat('not ');
var containsArrays = ramda.any(isArray$1);
var expandNegationObject = function expandNegationObject(negationObject) {
  return negationObject.not;
};
var containsMediaType = ramda.contains(ramda.__, ramda.values(MEDIA_TYPES));
var arrayContainsMediaType = ramda.any(containsMediaType);

var ensureMediaType = function ensureMediaType(defaultMediaType) {
  for (var _len = arguments.length, elements = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    elements[_key - 1] = arguments[_key];
  }

  return ramda.map(function (element) {
    if (isArray$1(element)) {
      // Only add prefix if no media type is declared
      if (arrayContainsMediaType(element)) {
        return element;
      }
      return [defaultMediaType].concat(toConsumableArray(element));
    }
    if (containsMediaType(element)) {
      return element;
    }
    return joinAnd([defaultMediaType, element]);
  })(elements);
};

var queryElementIsValidType = function queryElementIsValidType(element) {
  ramda.when(ramda.either(isNull, neither(isArrayOrString, isNegationObject)), composeError(queryElementIsValidTypeErrorMessage))(element);
};

var queryElementChildrenValidType = function queryElementChildrenValidType(element) {
  ramda.when(isArray$1, ramda.unless(ramda.all(function (child) {
    return ramda.either(isArrayOrString, isNegationObject)(child);
  }), composeError(queryChildElementIsValidTypeErrorMessage)))(element);
};

var elemementHasNoNestedArrays = function elemementHasNoNestedArrays(element) {
  ramda.when(isArray$1, ramda.when(containsArrays, composeError(queryNoNestedArraysErrorMessage)))(element);
};

var validateDefinition = function validateDefinition() {
  for (var _len2 = arguments.length, elements = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
    elements[_key2] = arguments[_key2];
  }

  ramda.forEach(function (element) {
    queryElementIsValidType(element);
    queryElementChildrenValidType(element);
    elemementHasNoNestedArrays(element);
  })(elements);
};

var renderFeature = function renderFeature(name, value) {
  return ramda.join('', ['(', nameValue([name, value]), ')']);
};

var renderQueryDefinition = function renderQueryDefinition() {
  for (var _len3 = arguments.length, elements = Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
    elements[_key3] = arguments[_key3];
  }

  validateDefinition.apply(undefined, toConsumableArray(elements));
  elements = ramda.map(ramda.when(isNegationObject, expandNegationObject))(elements);
  elements = ramda.map(ramda.when(isArray$1, joinAnd))(elements);
  return ramda.join(' ', [MEDIA_PREFIX, elements]);
};

var renderNotQueryDefinition = function renderNotQueryDefinition(defaultMediaType) {
  for (var _len4 = arguments.length, elements = Array(_len4 > 1 ? _len4 - 1 : 0), _key4 = 1; _key4 < _len4; _key4++) {
    elements[_key4 - 1] = arguments[_key4];
  }

  ramda.when(ramda.isEmpty, composeError(notNoElementsErrorMessage))(elements);
  validateDefinition.apply(undefined, toConsumableArray(elements));

  elements = ensureMediaType.apply(undefined, [defaultMediaType].concat(toConsumableArray(elements)));

  return ramda.compose(joinComma, ramda.map(ramda.compose(prefixWithNot, ramda.when(isArray$1, joinAnd))))(elements);
};

var buildLinearFeature = (function (name, possibleValues) {
  var allowNoArgument = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;
  return function (value) {
    ramda.unless(ramda.both(isUndefined, ramda.always(allowNoArgument)), ramda.partial(validateFeature, [name, possibleValues]))(value);
    return renderFeature(name, value);
  };
});

var toLinearFeatures = ramda.map(function (_ref) {
  var name = _ref.name,
      validValues = _ref.validValues,
      allowNoArgument = _ref.allowNoArgument;
  return defineProperty({}, camelcase(name), buildLinearFeature(name, validValues, allowNoArgument));
});

var buildLinearFeatures = (function () {
  return ramda.mergeAll(toLinearFeatures(LINEAR_FEATURES));
});

// -----------------------------------------------------------------------------
// Internal
// -----------------------------------------------------------------------------

// -----------------------------------------------------------------------------
// Exports
// -----------------------------------------------------------------------------

var propEqName = ramda.propEq('name');
var propName = ramda.prop('name');
var zipToNameAndValue = ramda.zipObj(['name', 'value']);
// TODO: This should be internal
var findBreakpointIndex = function findBreakpointIndex(breakpoint, breakpointsArray) {
  return ramda.findIndex(propEqName(breakpoint))(breakpointsArray);
};

var toBreakpointArray = ramda.compose(ramda.map(zipToNameAndValue), ramda.toPairs);

var getUpperLimit = function getUpperLimit(breakpointsArray, breakpoint) {
  var index = findBreakpointIndex(breakpoint, breakpointsArray);
  return ramda.compose(propName, ramda.nth(ramda.inc(index)))(breakpointsArray);
};

var buildFeatureItem = function buildFeatureItem(name, parser, config) {
  return function (breakpoint) {
    return renderFeature(name, parser(breakpoint, config));
  };
};

var nilValueAndAllowedToPass = function nilValueAndAllowedToPass(value, noArgs) {
  return ramda.both(ramda.isNil, ramda.always(noArgs))(value);
};

var buildRangedFeature = (function (name, valueRenderer) {
  var _o;

  var breakpoints = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

  var _ref = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {},
      _ref$defaultMediaType = _ref.defaultMediaType,
      defaultMediaType = _ref$defaultMediaType === undefined ? MEDIA_TYPES.SCREEN : _ref$defaultMediaType,
      _ref$onlyNamedBreakpo = _ref.onlyNamedBreakpoints,
      onlyNamedBreakpoints = _ref$onlyNamedBreakpo === undefined ? true : _ref$onlyNamedBreakpo,
      _ref$allowNoArgument = _ref.allowNoArgument,
      allowNoArgument = _ref$allowNoArgument === undefined ? false : _ref$allowNoArgument;

  var camelisedName = camelcase(name);

  var _rangedFeatureNamed = rangedFeatureNamed(name),
      validator = _rangedFeatureNamed.validator;

  // ---------------------------------------------------------------------------
  // UTILS
  // ---------------------------------------------------------------------------

  var getBreakpointNamed = function getBreakpointNamed(breakpoint) {
    if (ramda.isEmpty(breakpoints)) throwError(mssingBreakpointMapErrorMessage(camelisedName));
    var value = breakpoints[breakpoint];
    if (ramda.isNil(value)) throwError(missingBreakpointErrorMessage(breakpoint, name, breakpoints));
    return value;
  };

  var configuredValueRenderer = function configuredValueRenderer(value) {
    var _ref2 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
        _ref2$shouldSeparate = _ref2.shouldSeparate,
        shouldSeparate = _ref2$shouldSeparate === undefined ? false : _ref2$shouldSeparate,
        _ref2$noArgs = _ref2.noArgs,
        noArgs = _ref2$noArgs === undefined ? false : _ref2$noArgs;

    // TODO clean this up, but order here is vital - we need to return in this
    // order.
    if (nilValueAndAllowedToPass(value, noArgs)) {
      return valueRenderer(value, shouldSeparate);
    }

    if (onlyNamedBreakpoints) {
      value = getBreakpointNamed(value);
      return valueRenderer(value, shouldSeparate);
    }

    validator.validate(value);
    return valueRenderer(value, shouldSeparate);
  };

  var defaultAPIConfig = { mediaType: defaultMediaType };

  var orderedBreakpoints = toBreakpointArray(breakpoints);
  var indexOfBreakpointNamed = function indexOfBreakpointNamed(value) {
    return ramda.findIndex(value, orderedBreakpoints);
  };

  var nextBreakpointAboveNamed = function nextBreakpointAboveNamed(value) {
    return getUpperLimit(orderedBreakpoints, value);
  };

  // ---------------------------------------------------------------------------
  // Features
  // ---------------------------------------------------------------------------

  var feature = buildFeatureItem(name, configuredValueRenderer, {
    noArgs: allowNoArgument
  });
  var minFeature = buildFeatureItem('min-' + name, configuredValueRenderer);
  var maxFeature = buildFeatureItem('max-' + name, configuredValueRenderer, {
    shouldSeparate: true
  });

  // ---------------------------------------------------------------------------
  // Feature Helpers
  // ---------------------------------------------------------------------------

  var aboveFeature = function aboveFeature(from) {
    return minFeature(from);
  };

  var belowFeature = function belowFeature(to) {
    return maxFeature(to);
  };

  var betweenFeatures = function betweenFeatures(from, to) {
    if (from === to) throwError(sameBreakpointsForBetweenErrorMessage(from));
    var fromIndex = indexOfBreakpointNamed(propEqName(from));
    var toIndex = indexOfBreakpointNamed(propEqName(to));

    var _ref3 = fromIndex < toIndex ? [from, to] : [to, from],
        _ref4 = slicedToArray(_ref3, 2),
        lower = _ref4[0],
        higher = _ref4[1];

    return joinAnd([minFeature(lower), maxFeature(higher)]);
  };

  var atFeatureBreakpoint = function atFeatureBreakpoint(breakpoint) {
    var config = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : defaultAPIConfig;

    var breakpointAbove = nextBreakpointAboveNamed(breakpoint);
    if (breakpointAbove) {
      return betweenFeatures(breakpoint, breakpointAbove, config);
    }
    return aboveFeature(breakpoint, config);
  };

  var atFeature = function atFeature(breakpoint) {
    return feature(breakpoint);
  };

  var titleizedName = name[0].toUpperCase() + camelcase(name.slice(1));

  var o = (_o = {}, defineProperty(_o, camelcase(name), feature), defineProperty(_o, 'min' + [titleizedName], minFeature), defineProperty(_o, 'max' + [titleizedName], maxFeature), defineProperty(_o, 'above' + [titleizedName], aboveFeature), defineProperty(_o, 'below' + [titleizedName], belowFeature), defineProperty(_o, 'between' + [titleizedName] + 's', betweenFeatures), defineProperty(_o, 'at' + [titleizedName] + 'Breakpoint', atFeatureBreakpoint), defineProperty(_o, 'at' + [titleizedName], atFeature), _o);

  return o;
});

var build = function build(globalConfig, breakpoints, item) {
  breakpoints = ramda.defaultTo({})(breakpoints);
  var x = buildRangedFeature(item.name, item.valueRenderer(globalConfig), breakpoints[camelcase(item.name)], ramda.merge(globalConfig, item.config));
  return x;
};

var buildRangeFeatures = (function (breakpoints, globalConfig) {
  return ramda.mergeAll(ramda.map(ramda.curry(build)(globalConfig, breakpoints))(RANGED_FEATURES));
});

var _templateObject = taggedTemplateLiteral(['\n  ', ' {\n    ', ';\n  }\n'], ['\n  ', ' {\n    ', ';\n  }\n']);

var renderQuery = (function (definition, content) {
  return styledComponents.css(_templateObject, definition, content);
});

var defaultConfig = {
  baseFontSize: 16,
  defaultMediaType: MEDIA_TYPES.SCREEN,
  dimensionsUnit: UNITS.DIMENSIONS.EM,
  shouldSeparateQueries: true,
  onlyNamedBreakpoints: true
};

var validateConfigArgs = function validateConfigArgs(breakpoints, config) {
  validateConfig(config);
  if (ramda.complement(isUndefined)(breakpoints)) validateBreakpointMap(breakpoints);
  validateBreakpointSets(breakpoints);
};

// Don't expand config vars as we need to pass a single config object around.
var configure = function configure(breakpoints) {
  var config = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var originalMQ = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;

  var configWithDefaults = ramda.merge(defaultConfig, config);

  validateConfigArgs(breakpoints, configWithDefaults);
  // Ensure we have a unitless value stored for baseFontSize
  if (isNumberWithDimensionsUnit(configWithDefaults.baseFontSize)) {
    configWithDefaults.baseFontSize = unitedDimensionToUnitlessPixelValue(configWithDefaults.baseFontSize, 16);
  }
  // ---------------------------------------------------------------------------
  // API
  // ---------------------------------------------------------------------------

  var tweak = function tweak(mq, tweakpoints) {
    if (breakpoints) validateBreakpointMap(tweakpoints);
    validateBreakpointSets(tweakpoints);
    var mergedBreakpoints = ramda.mergeDeepLeft(breakpoints, tweakpoints);
    return configure(mergedBreakpoints, configWithDefaults, mq);
  };

  var query = function query() {
    for (var _len = arguments.length, elements = Array(_len), _key = 0; _key < _len; _key++) {
      elements[_key] = arguments[_key];
    }

    if (ramda.isEmpty(elements)) throwError(queryNoElementsErrorMessage());

    return function (stringParts) {
      for (var _len2 = arguments.length, interpolationValues = Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
        interpolationValues[_key2 - 1] = arguments[_key2];
      }

      return renderQuery(renderQueryDefinition.apply(undefined, elements), styledComponents.css.apply(undefined, [stringParts].concat(interpolationValues)));
    };
  };

  var not = function not() {
    for (var _len3 = arguments.length, elements = Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
      elements[_key3] = arguments[_key3];
    }

    return {
      not: renderNotQueryDefinition.apply(undefined, [configWithDefaults.defaultMediaType].concat(elements))
    };
  };

  // ---------------------------------------------------------------------------
  // Export
  // ---------------------------------------------------------------------------

  var o = _extends({
    mediaType: buildMediaType(configWithDefaults.defaultMediaType)
  }, buildLinearFeatures(), buildRangeFeatures(breakpoints, configWithDefaults), {
    query: query,
    not: not,
    untweaked: function untweaked() {
      if (!originalMQ) {
        throwError(noUntweakedErrorMessage());
      }
      return originalMQ;
    }
  });
  o.tweak = ramda.partial(tweak, [o]);
  return o;
};

var mq$1 = {
  configure: configure
};

return mq$1;

})));
