'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.rangedFeatureNamed = exports.rangedFeatureNames = exports.RANGED_FEATURES = exports.LINEAR_FEATURES = exports.DISPLAY_MODE = exports.COLOR_GAMUT = exports.OVERFLOW_INLINE = exports.OVERFLOW_BLOCK = exports.UPDATE = exports.GRID = exports.SCAN = exports.ORIENTATION = undefined;

var _ramda = require('ramda');

var _camelcase = require('camelcase');

var _camelcase2 = _interopRequireDefault(_camelcase);

var _dasherize = require('dasherize');

var _dasherize2 = _interopRequireDefault(_dasherize);

var _dimensionsValueRenderer = require('./renderers/valueRenderers/dimensionsValueRenderer');

var _dimensionsValueRenderer2 = _interopRequireDefault(_dimensionsValueRenderer);

var _resolutionValueRenderer = require('./renderers/valueRenderers/resolutionValueRenderer');

var _resolutionValueRenderer2 = _interopRequireDefault(_resolutionValueRenderer);

var _aspectRatioValueRenderer = require('./renderers/valueRenderers/aspectRatioValueRenderer');

var _aspectRatioValueRenderer2 = _interopRequireDefault(_aspectRatioValueRenderer);

var _colorValueRenderer = require('./renderers/valueRenderers/colorValueRenderer');

var _colorValueRenderer2 = _interopRequireDefault(_colorValueRenderer);

var _monochromeValueRenderer = require('./renderers/valueRenderers/monochromeValueRenderer');

var _monochromeValueRenderer2 = _interopRequireDefault(_monochromeValueRenderer);

var _dimensionValidator = require('./validators/features/dimensionValidator');

var _dimensionValidator2 = _interopRequireDefault(_dimensionValidator);

var _resolutionValidator = require('./validators/features/resolutionValidator');

var _resolutionValidator2 = _interopRequireDefault(_resolutionValidator);

var _aspectRatioValidator = require('./validators/features/aspectRatioValidator');

var _aspectRatioValidator2 = _interopRequireDefault(_aspectRatioValidator);

var _colorValidator = require('./validators/features/colorValidator');

var _colorValidator2 = _interopRequireDefault(_colorValidator);

var _monochromeValidator = require('./validators/features/monochromeValidator');

var _monochromeValidator2 = _interopRequireDefault(_monochromeValidator);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var ORIENTATION = exports.ORIENTATION = Object.freeze(['portrait', 'landscape']);
var SCAN = exports.SCAN = Object.freeze(['interlace', 'progressive']);
var GRID = exports.GRID = Object.freeze([0, 1]);
var UPDATE = exports.UPDATE = Object.freeze(['none', 'slow', 'fast']);
var OVERFLOW_BLOCK = exports.OVERFLOW_BLOCK = Object.freeze(['none', 'scroll', 'optional-paged']);
var OVERFLOW_INLINE = exports.OVERFLOW_INLINE = Object.freeze(['none', 'scroll']);
var COLOR_GAMUT = exports.COLOR_GAMUT = Object.freeze(['srgb', 'p3', 'rec2020']);
var DISPLAY_MODE = exports.DISPLAY_MODE = Object.freeze(['fullscreen', 'standalone', 'minimal-ui', 'browser']);

var LINEAR_FEATURES = exports.LINEAR_FEATURES = [{
  name: 'orientation',
  validValues: ORIENTATION
}, {
  name: 'scan',
  validValues: SCAN
}, {
  name: 'grid',
  validValues: GRID,
  allowNoArgument: true
}, {
  name: 'update',
  validValues: UPDATE,
  allowNoArgument: true
}, {
  name: 'overflow-block',
  validValues: OVERFLOW_BLOCK
}, {
  name: 'overflow-inline',
  validValues: OVERFLOW_INLINE
}, {
  name: 'color-gamut',
  validValues: COLOR_GAMUT
}, {
  name: 'display-mode',
  validValues: DISPLAY_MODE
}];

var RANGED_FEATURES = exports.RANGED_FEATURES = [{
  name: 'width',
  valueRenderer: _dimensionsValueRenderer2.default,
  validator: _dimensionValidator2.default
}, {
  name: 'height',
  valueRenderer: _dimensionsValueRenderer2.default,
  validator: _dimensionValidator2.default
}, {
  name: 'resolution',
  valueRenderer: _resolutionValueRenderer2.default,
  validator: _resolutionValidator2.default
}, {
  name: 'aspect-ratio',
  valueRenderer: _aspectRatioValueRenderer2.default,
  validator: _aspectRatioValidator2.default
}, {
  name: 'color',
  valueRenderer: _colorValueRenderer2.default,
  validator: _colorValidator2.default,
  config: {
    allowNoArgument: true
  }
}, {
  name: 'color-index',
  valueRenderer: _colorValueRenderer2.default,
  validator: _colorValidator2.default,
  config: {
    allowNoArgument: true
  }
}, {
  name: 'monochrome',
  valueRenderer: _monochromeValueRenderer2.default,
  validator: _monochromeValidator2.default,
  config: {
    allowNoArgument: true
  }
}];

var rangedFeatureNames = exports.rangedFeatureNames = (0, _ramda.map)((0, _ramda.compose)(_camelcase2.default, (0, _ramda.prop)('name')))(RANGED_FEATURES);

var rangedFeatureNamed = exports.rangedFeatureNamed = (0, _ramda.compose)((0, _ramda.find)(_ramda.__, RANGED_FEATURES), (0, _ramda.propEq)('name'), _dasherize2.default);