'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _ramda = require('ramda');

var _camelcase = require('camelcase');

var _camelcase2 = _interopRequireDefault(_camelcase);

var _const = require('../const');

var _errors = require('../errors');

var _breakpoints = require('../utils/breakpoints');

var _features = require('../features');

var _queryRenderer = require('../renderers/cssRenderers/queryRenderer');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var buildFeatureItem = function buildFeatureItem(name, parser, config) {
  return function (breakpoint) {
    return (0, _queryRenderer.renderFeature)(name, parser(breakpoint, config));
  };
};

var nilValueAndAllowedToPass = function nilValueAndAllowedToPass(value, noArgs) {
  return (0, _ramda.both)(_ramda.isNil, (0, _ramda.always)(noArgs))(value);
};

exports.default = function (name, valueRenderer) {
  var _o;

  var breakpoints = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

  var _ref = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {},
      _ref$defaultMediaType = _ref.defaultMediaType,
      defaultMediaType = _ref$defaultMediaType === undefined ? _const.MEDIA_TYPES.SCREEN : _ref$defaultMediaType,
      _ref$onlyNamedBreakpo = _ref.onlyNamedBreakpoints,
      onlyNamedBreakpoints = _ref$onlyNamedBreakpo === undefined ? true : _ref$onlyNamedBreakpo,
      _ref$allowNoArgument = _ref.allowNoArgument,
      allowNoArgument = _ref$allowNoArgument === undefined ? false : _ref$allowNoArgument;

  var camelisedName = (0, _camelcase2.default)(name);

  var _rangedFeatureNamed = (0, _features.rangedFeatureNamed)(name),
      validator = _rangedFeatureNamed.validator;

  // ---------------------------------------------------------------------------
  // UTILS
  // ---------------------------------------------------------------------------

  var getBreakpointNamed = function getBreakpointNamed(breakpoint) {
    if ((0, _ramda.isEmpty)(breakpoints)) (0, _errors.throwError)((0, _errors.mssingBreakpointMapErrorMessage)(camelisedName));
    var value = breakpoints[breakpoint];
    if ((0, _ramda.isNil)(value)) (0, _errors.throwError)((0, _errors.missingBreakpointErrorMessage)(breakpoint, name, breakpoints));
    return value;
  };

  var configuredValueRenderer = function configuredValueRenderer(value) {
    var _ref2 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
        _ref2$shouldSeparate = _ref2.shouldSeparate,
        shouldSeparate = _ref2$shouldSeparate === undefined ? false : _ref2$shouldSeparate,
        _ref2$noArgs = _ref2.noArgs,
        noArgs = _ref2$noArgs === undefined ? false : _ref2$noArgs;

    // TODO clean this up, but order here is vital - we need to return in this
    // order.
    if (nilValueAndAllowedToPass(value, noArgs)) {
      return valueRenderer(value, shouldSeparate);
    }

    if (onlyNamedBreakpoints) {
      value = getBreakpointNamed(value);
      return valueRenderer(value, shouldSeparate);
    }

    validator.validate(value);
    return valueRenderer(value, shouldSeparate);
  };

  var defaultAPIConfig = { mediaType: defaultMediaType };

  var orderedBreakpoints = (0, _breakpoints.toBreakpointArray)(breakpoints);
  var indexOfBreakpointNamed = function indexOfBreakpointNamed(value) {
    return (0, _ramda.findIndex)(value, orderedBreakpoints);
  };

  var nextBreakpointAboveNamed = function nextBreakpointAboveNamed(value) {
    return (0, _breakpoints.getUpperLimit)(orderedBreakpoints, value);
  };

  // ---------------------------------------------------------------------------
  // Features
  // ---------------------------------------------------------------------------

  var feature = buildFeatureItem(name, configuredValueRenderer, {
    noArgs: allowNoArgument
  });
  var minFeature = buildFeatureItem('min-' + name, configuredValueRenderer);
  var maxFeature = buildFeatureItem('max-' + name, configuredValueRenderer, {
    shouldSeparate: true
  });

  // ---------------------------------------------------------------------------
  // Feature Helpers
  // ---------------------------------------------------------------------------

  var aboveFeature = function aboveFeature(from) {
    return minFeature(from);
  };

  var belowFeature = function belowFeature(to) {
    return maxFeature(to);
  };

  var betweenFeatures = function betweenFeatures(from, to) {
    if (from === to) (0, _errors.throwError)((0, _errors.sameBreakpointsForBetweenErrorMessage)(from));
    var fromIndex = indexOfBreakpointNamed((0, _breakpoints.propEqName)(from));
    var toIndex = indexOfBreakpointNamed((0, _breakpoints.propEqName)(to));

    var _ref3 = fromIndex < toIndex ? [from, to] : [to, from],
        _ref4 = _slicedToArray(_ref3, 2),
        lower = _ref4[0],
        higher = _ref4[1];

    return (0, _queryRenderer.joinAnd)([minFeature(lower), maxFeature(higher)]);
  };

  var atFeatureBreakpoint = function atFeatureBreakpoint(breakpoint) {
    var config = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : defaultAPIConfig;

    var breakpointAbove = nextBreakpointAboveNamed(breakpoint);
    if (breakpointAbove) {
      return betweenFeatures(breakpoint, breakpointAbove, config);
    }
    return aboveFeature(breakpoint, config);
  };

  var atFeature = function atFeature(breakpoint) {
    return feature(breakpoint);
  };

  var titleizedName = name[0].toUpperCase() + (0, _camelcase2.default)(name.slice(1));

  var o = (_o = {}, _defineProperty(_o, (0, _camelcase2.default)(name), feature), _defineProperty(_o, 'min' + [titleizedName], minFeature), _defineProperty(_o, 'max' + [titleizedName], maxFeature), _defineProperty(_o, 'above' + [titleizedName], aboveFeature), _defineProperty(_o, 'below' + [titleizedName], belowFeature), _defineProperty(_o, 'between' + [titleizedName] + 's', betweenFeatures), _defineProperty(_o, 'at' + [titleizedName] + 'Breakpoint', atFeatureBreakpoint), _defineProperty(_o, 'at' + [titleizedName], atFeature), _o);

  return o;
};