'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _styledComponents = require('styled-components');

var _ramda = require('ramda');

var _buildMediaType = require('./features/buildMediaType');

var _buildMediaType2 = _interopRequireDefault(_buildMediaType);

var _buildLinearFeatures = require('./features/buildLinearFeatures');

var _buildLinearFeatures2 = _interopRequireDefault(_buildLinearFeatures);

var _buildRangedFeatures = require('./features/buildRangedFeatures');

var _buildRangedFeatures2 = _interopRequireDefault(_buildRangedFeatures);

var _validations = require('./validations');

var _predicates = require('./utils/predicates');

var _units = require('./utils/units');

var _const = require('./const');

var _styledComponentsRenderer = require('./renderers/cssRenderers/styledComponentsRenderer');

var _styledComponentsRenderer2 = _interopRequireDefault(_styledComponentsRenderer);

var _queryRenderer = require('./renderers/cssRenderers/queryRenderer');

var _errors = require('./errors');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var defaultConfig = {
  baseFontSize: 16,
  defaultMediaType: _const.MEDIA_TYPES.SCREEN,
  dimensionsUnit: _const.UNITS.DIMENSIONS.EM,
  shouldSeparateQueries: true,
  onlyNamedBreakpoints: true
};

var validateConfigArgs = function validateConfigArgs(breakpoints, config) {
  (0, _validations.validateConfig)(config);
  if ((0, _ramda.complement)(_predicates.isUndefined)(breakpoints)) (0, _validations.validateBreakpointMap)(breakpoints);
  (0, _validations.validateBreakpointSets)(breakpoints);
};

// Don't expand config vars as we need to pass a single config object around.
var configure = function configure(breakpoints) {
  var config = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var originalMQ = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;

  var configWithDefaults = (0, _ramda.merge)(defaultConfig, config);

  validateConfigArgs(breakpoints, configWithDefaults);
  // Ensure we have a unitless value stored for baseFontSize
  if ((0, _predicates.isNumberWithDimensionsUnit)(configWithDefaults.baseFontSize)) {
    configWithDefaults.baseFontSize = (0, _units.unitedDimensionToUnitlessPixelValue)(configWithDefaults.baseFontSize, 16);
  }
  // ---------------------------------------------------------------------------
  // API
  // ---------------------------------------------------------------------------

  var tweak = function tweak(mq, tweakpoints) {
    if (breakpoints) (0, _validations.validateBreakpointMap)(tweakpoints);
    (0, _validations.validateBreakpointSets)(tweakpoints);
    var mergedBreakpoints = (0, _ramda.mergeDeepLeft)(breakpoints, tweakpoints);
    return configure(mergedBreakpoints, configWithDefaults, mq);
  };

  var query = function query() {
    for (var _len = arguments.length, elements = Array(_len), _key = 0; _key < _len; _key++) {
      elements[_key] = arguments[_key];
    }

    if ((0, _ramda.isEmpty)(elements)) (0, _errors.throwError)((0, _errors.queryNoElementsErrorMessage)());

    return function (stringParts) {
      for (var _len2 = arguments.length, interpolationValues = Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
        interpolationValues[_key2 - 1] = arguments[_key2];
      }

      return (0, _styledComponentsRenderer2.default)(_queryRenderer.renderQueryDefinition.apply(undefined, elements), _styledComponents.css.apply(undefined, [stringParts].concat(interpolationValues)));
    };
  };

  var not = function not() {
    for (var _len3 = arguments.length, elements = Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
      elements[_key3] = arguments[_key3];
    }

    return {
      not: _queryRenderer.renderNotQueryDefinition.apply(undefined, [configWithDefaults.defaultMediaType].concat(elements))
    };
  };

  // ---------------------------------------------------------------------------
  // Export
  // ---------------------------------------------------------------------------

  var o = _extends({
    mediaType: (0, _buildMediaType2.default)(configWithDefaults.defaultMediaType)
  }, (0, _buildLinearFeatures2.default)(), (0, _buildRangedFeatures2.default)(breakpoints, configWithDefaults), {
    query: query,
    not: not,
    untweaked: function untweaked() {
      if (!originalMQ) {
        (0, _errors.throwError)((0, _errors.noUntweakedErrorMessage)());
      }
      return originalMQ;
    }
  });
  o.tweak = (0, _ramda.partial)(tweak, [o]);
  return o;
};

exports.default = {
  configure: configure
};