'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.renderNotQueryDefinition = exports.renderQueryDefinition = exports.renderFeature = exports.ensureMediaType = exports.joinAnd = undefined;

var _ramda = require('ramda');

var _predicates = require('../../utils/predicates');

var _const = require('../../const');

var _errors = require('../../errors');

var _logic = require('../../utils/logic');

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var nameValue = (0, _ramda.compose)((0, _ramda.join)(': '), (0, _ramda.reject)(_ramda.isNil));
var isArrayOrString = (0, _ramda.either)(_predicates.isArray, _predicates.isString);
var isNegationObject = (0, _ramda.both)(_predicates.isObject, (0, _ramda.has)('not'));

var joinAnd = exports.joinAnd = (0, _ramda.join)(' and ');
var joinComma = (0, _ramda.join)(', ');
var prefixWithNot = (0, _ramda.concat)('not ');
var containsArrays = (0, _ramda.any)(_predicates.isArray);
var expandNegationObject = function expandNegationObject(negationObject) {
  return negationObject.not;
};
var containsMediaType = (0, _ramda.contains)(_ramda.__, (0, _ramda.values)(_const.MEDIA_TYPES));
var arrayContainsMediaType = (0, _ramda.any)(containsMediaType);

var ensureMediaType = exports.ensureMediaType = function ensureMediaType(defaultMediaType) {
  for (var _len = arguments.length, elements = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    elements[_key - 1] = arguments[_key];
  }

  return (0, _ramda.map)(function (element) {
    if ((0, _predicates.isArray)(element)) {
      // Only add prefix if no media type is declared
      if (arrayContainsMediaType(element)) {
        return element;
      }
      return [defaultMediaType].concat(_toConsumableArray(element));
    }
    if (containsMediaType(element)) {
      return element;
    }
    return joinAnd([defaultMediaType, element]);
  })(elements);
};

var queryElementIsValidType = function queryElementIsValidType(element) {
  (0, _ramda.when)((0, _ramda.either)(_predicates.isNull, (0, _logic.neither)(isArrayOrString, isNegationObject)), (0, _errors.composeError)(_errors.queryElementIsValidTypeErrorMessage))(element);
};

var queryElementChildrenValidType = function queryElementChildrenValidType(element) {
  (0, _ramda.when)(_predicates.isArray, (0, _ramda.unless)((0, _ramda.all)(function (child) {
    return (0, _ramda.either)(isArrayOrString, isNegationObject)(child);
  }), (0, _errors.composeError)(_errors.queryChildElementIsValidTypeErrorMessage)))(element);
};

var elemementHasNoNestedArrays = function elemementHasNoNestedArrays(element) {
  (0, _ramda.when)(_predicates.isArray, (0, _ramda.when)(containsArrays, (0, _errors.composeError)(_errors.queryNoNestedArraysErrorMessage)))(element);
};

var validateDefinition = function validateDefinition() {
  for (var _len2 = arguments.length, elements = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
    elements[_key2] = arguments[_key2];
  }

  (0, _ramda.forEach)(function (element) {
    queryElementIsValidType(element);
    queryElementChildrenValidType(element);
    elemementHasNoNestedArrays(element);
  })(elements);
};

var renderFeature = exports.renderFeature = function renderFeature(name, value) {
  return (0, _ramda.join)('', ['(', nameValue([name, value]), ')']);
};

var renderQueryDefinition = exports.renderQueryDefinition = function renderQueryDefinition() {
  for (var _len3 = arguments.length, elements = Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
    elements[_key3] = arguments[_key3];
  }

  validateDefinition.apply(undefined, _toConsumableArray(elements));
  elements = (0, _ramda.map)((0, _ramda.when)(isNegationObject, expandNegationObject))(elements);
  elements = (0, _ramda.map)((0, _ramda.when)(_predicates.isArray, joinAnd))(elements);
  return (0, _ramda.join)(' ', [_const.MEDIA_PREFIX, elements]);
};

var renderNotQueryDefinition = exports.renderNotQueryDefinition = function renderNotQueryDefinition(defaultMediaType) {
  for (var _len4 = arguments.length, elements = Array(_len4 > 1 ? _len4 - 1 : 0), _key4 = 1; _key4 < _len4; _key4++) {
    elements[_key4 - 1] = arguments[_key4];
  }

  (0, _ramda.when)(_ramda.isEmpty, (0, _errors.composeError)(_errors.notNoElementsErrorMessage))(elements);
  validateDefinition.apply(undefined, _toConsumableArray(elements));

  elements = ensureMediaType.apply(undefined, [defaultMediaType].concat(_toConsumableArray(elements)));

  return (0, _ramda.compose)(joinComma, (0, _ramda.map)((0, _ramda.compose)(prefixWithNot, (0, _ramda.when)(_predicates.isArray, joinAnd))))(elements);
};