'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.areMediaTypesValid = exports.isMediaTypeValid = exports.isDimensionsUnitValid = exports.doesListIncludeValue = exports.isPositiveNumberWithDimensionsUnit = exports.isPositiveNumberWithResolutionUnit = exports.isNumberWithResolutionUnit = exports.isNumberWithDimensionsUnit = exports.isPositiveNumberWithPixelUnit = exports.isNumericPartOfUnitValuePositive = exports.isNumberWithUnit = exports.isPositiveIntegerOrZero = exports.isPositiveInteger = exports.isPositiveNumberOrZero = exports.isPositiveNumber = exports.isRatioString = exports.isPopulatedObject = exports.isUndefined = exports.isNull = exports.isArray = exports.isString = exports.isObject = exports.isNumber = exports.isBoolean = undefined;

var _ramda = require('ramda');

var _units = require('./units');

var _const = require('../const');

var isBoolean = exports.isBoolean = (0, _ramda.is)(Boolean);
var isNumber = exports.isNumber = (0, _ramda.both)((0, _ramda.is)(Number), (0, _ramda.complement)((0, _ramda.equals)(NaN)));
var isObject = exports.isObject = (0, _ramda.is)(Object);
var isString = exports.isString = (0, _ramda.is)(String);
var isArray = exports.isArray = (0, _ramda.is)(Array);
var isNull = exports.isNull = function isNull(value) {
  return value === null;
};
var isUndefined = exports.isUndefined = function isUndefined(value) {
  return value === undefined;
};

var isPopulatedObject = exports.isPopulatedObject = (0, _ramda.both)((0, _ramda.complement)(_ramda.isEmpty), isObject);
var isRatioString = exports.isRatioString = (0, _ramda.test)(/^[1-9]+[0-9]* ?\/ ?[1-9]+[0-9]*$/);
var isPositiveNumber = exports.isPositiveNumber = (0, _ramda.both)(isNumber, (0, _ramda.gt)(_ramda.__, 0));
var isPositiveNumberOrZero = exports.isPositiveNumberOrZero = (0, _ramda.both)(isNumber, (0, _ramda.gte)(_ramda.__, 0));
var isPositiveInteger = exports.isPositiveInteger = (0, _ramda.both)(isPositiveNumber, Number.isInteger);
var isPositiveIntegerOrZero = exports.isPositiveIntegerOrZero = (0, _ramda.both)(isPositiveNumberOrZero, Number.isInteger);
var isNumberWithUnit = exports.isNumberWithUnit = (0, _ramda.curry)(function (units, value) {
  var regex = '^-?\\d+(?:.\\d+)?(?:' + (0, _ramda.join)('|', units) + ')$';
  return new RegExp(regex).test(value);
});
var isNumericPartOfUnitValuePositive = exports.isNumericPartOfUnitValuePositive = (0, _ramda.compose)(isPositiveNumber, _units.numericPartOfUnitedNumber);
var isPositiveNumberWithPixelUnit = exports.isPositiveNumberWithPixelUnit = (0, _ramda.both)(isNumberWithUnit([_const.DIMENSIONS_UNITS.PX]), isNumericPartOfUnitValuePositive);
var isNumberWithDimensionsUnit = exports.isNumberWithDimensionsUnit = isNumberWithUnit((0, _ramda.values)(_const.DIMENSIONS_UNITS));
var isNumberWithResolutionUnit = exports.isNumberWithResolutionUnit = isNumberWithUnit([_const.RESOLUTION_UNIT]);
var isPositiveNumberWithResolutionUnit = exports.isPositiveNumberWithResolutionUnit = (0, _ramda.both)(isNumberWithResolutionUnit, isNumericPartOfUnitValuePositive);
var isPositiveNumberWithDimensionsUnit = exports.isPositiveNumberWithDimensionsUnit = (0, _ramda.both)(isNumberWithDimensionsUnit, isNumericPartOfUnitValuePositive);
var doesListIncludeValue = exports.doesListIncludeValue = function doesListIncludeValue(list) {
  return (0, _ramda.contains)(_ramda.__, (0, _ramda.values)(list));
};
var isDimensionsUnitValid = exports.isDimensionsUnitValid = (0, _ramda.contains)(_ramda.__, (0, _ramda.values)(_const.DIMENSIONS_UNITS));
var isMediaTypeValid = exports.isMediaTypeValid = (0, _ramda.flip)(_ramda.contains)((0, _ramda.values)(_const.MEDIA_TYPES));
var areMediaTypesValid = exports.areMediaTypesValid = (0, _ramda.both)((0, _ramda.all)(isMediaTypeValid), (0, _ramda.complement)(_ramda.isEmpty));