'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.validateFeature = exports.validateConfig = exports.validateBreakpointSets = exports.validateBreakpointMap = exports.validateMediaTypes = undefined;

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _ramda = require('ramda');

var _features = require('./features');

var _array = require('./utils/array');

var _predicates = require('./utils/predicates');

var _errors = require('./errors');

// TODO: Can't move these to predicates as they import helper from features and
// this causes an acyclical dependency between features -> validators ->
// predicates.
var isBreakpointSetNameValid = (0, _ramda.contains)(_ramda.__, _features.rangedFeatureNames);
var areBreakpointSetNamesValid = (0, _ramda.all)(isBreakpointSetNameValid);

var validate = function validate(predicate, errorMessage) {
  return function (value) {
    (0, _ramda.unless)(predicate, (0, _errors.composeError)(errorMessage))(value);
  };
};

// Validate a map of breakpoint sets.

// -----------------------------------------------------------------------------
// Exports
// -----------------------------------------------------------------------------

var validateBreakpointSetValues = function validateBreakpointSetValues(name, breakpointSet) {
  var _rangedFeatureNamed = (0, _features.rangedFeatureNamed)(name),
      validator = _rangedFeatureNamed.validator;

  validate((0, _ramda.compose)((0, _ramda.all)(validator.validate), _ramda.values), (0, _ramda.compose)((0, _ramda.curry)(_errors.invalidBreakpointSetValueErrorMessage)(validator.message), _ramda.values))(breakpointSet);
};

var validateBreakpointSetObject = function validateBreakpointSetObject(name, breakpointSet) {
  validate(_predicates.isPopulatedObject, (0, _ramda.always)((0, _errors.emptyBreakpointSetErrorMessage)(name)))(breakpointSet);
};

// Validate a set of breakpoints.
var validateBreakpointSet = function validateBreakpointSet(name, breakpointSet) {
  validateBreakpointSetObject(name, breakpointSet);
  validateBreakpointSetValues(name, breakpointSet);
};

var validateMediaTypes = exports.validateMediaTypes = validate(_predicates.areMediaTypesValid, _errors.invalidMediaTypeErrorMessage);

var validateBreakpointSetNames = validate((0, _ramda.compose)(areBreakpointSetNamesValid, _ramda.keys), _errors.invalidBreakpointNamesErrorMessage);

var validateBreakpointObject = function validateBreakpointObject(breakpointMap) {
  validate((0, _ramda.both)((0, _ramda.complement)(_predicates.isArray), _predicates.isObject), _errors.invalidBreakpointsErrorMessage)(breakpointMap);
  validate(_predicates.isPopulatedObject, _errors.emptyBreakpointMapErrorMessage)(breakpointMap);
};

var validateBreakpointMap = exports.validateBreakpointMap = function validateBreakpointMap(breakpoints) {
  validateBreakpointObject(breakpoints);
  validateBreakpointSetNames(breakpoints);
};

var validateBreakpointSets = exports.validateBreakpointSets = (0, _ramda.compose)((0, _ramda.map)(function (_ref) {
  var _ref2 = _slicedToArray(_ref, 2),
      name = _ref2[0],
      value = _ref2[1];

  return validateBreakpointSet(name, value);
}), _ramda.toPairs);

var validateConfig = exports.validateConfig = function validateConfig(_ref3) {
  var baseFontSize = _ref3.baseFontSize,
      defaultMediaType = _ref3.defaultMediaType,
      dimensionsUnit = _ref3.dimensionsUnit,
      shouldSeparateQueries = _ref3.shouldSeparateQueries;

  validate((0, _ramda.either)(_predicates.isPositiveNumber, _predicates.isPositiveNumberWithPixelUnit), _errors.invalidBaseFontSizeErrorMessage)(baseFontSize);

  validate((0, _ramda.anyPass)([_predicates.isNull, (0, _ramda.compose)(_predicates.areMediaTypesValid, _array.ensureArray)]), _errors.invalidDefaultMediaTypeErrorMessage)(defaultMediaType);

  validate(_predicates.isDimensionsUnitValid, _errors.invalidDimensionsUnitErrorMessage)(dimensionsUnit);
  validate(_predicates.isBoolean, _errors.shouldSeparateQueriesErrorMessage)(shouldSeparateQueries);
};

var validateFeature = exports.validateFeature = function validateFeature(name, possibleValues, value) {
  validate((0, _predicates.doesListIncludeValue)(possibleValues), (0, _errors.invalidFeatureErrorMessage)(name, possibleValues))(value);
};